/* XDCtools wb_t@C */
#include <xdc/std.h>
#include <xdc/runtime/System.h>

/* BIOS wb_t@C */
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Clock.h>
#include <ti/sysbios/hal/Seconds.h>

/* C wb_t@C */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/* hCowb_t@C */
#include <ti/drivers/GPIO.h>
#include <ti/drivers/PWM.h>
#include <ti/drivers/SDSPI.h>

/* NDK wb_t@C */
#include <ti/ndk/inc/netmain.h>

/* {[hwb_t@C */
#include "Board.h"

/* ADCwb_t@C */
#include "ADC.h"

/* GLCD wb_t@C */
#include "GLCD.h"

/* [U[`}N */
#define CONSOLE(...) do { System_printf(__VA_ARGS__); System_flush(); } while(0)
#define SLEEP(X) Task_sleep((X)*1000/Clock_tickPeriod)

/*
 * [U[^XN̗Dx
 */
#define TASK_LED_PRIO 1
#define TASK_GLCD_PRIO 9
#define TASK_FATFS_PRIO 10
#define TASK_LDM_PRIO 11
#define TASK_SNTP_PRIO 6

/*
 * [U[X^bÑTCY
 */
#define TASK_LED_STACK 512
#define TASK_GLCD_STACK 512
#define TASK_FATFS_STACK 1024
#define TASK_LDM_STACK 512
#define TASK_SNTP_STACK 2048

/*
 * [U[^XN̍\
 */
Task_Struct taskLEDStruct;
Task_Struct taskGLCDStruct;
Task_Struct taskFATFSStruct;
Task_Struct taskLDMStruct;
Task_Struct taskSNTPStruct;

/*
 * [U[^XÑX^bN
 */
Char taskLEDStack[TASK_LED_STACK];
Char taskGLCDStack[TASK_GLCD_STACK];
Char taskFATFSStack[TASK_FATFS_STACK];
Char taskLDMStack[TASK_LDM_STACK];
Char taskSNTPStack[TASK_SNTP_STACK];

/*
 * [U[ϐ
 */
uint32_t IP;       // IPAhXi[pϐ
char lockSD;      // SDJ[hrpϐ
char toggleLDM;   // LDM̃gO
char charLDM;     // LDMɕ\镶
PWM_Handle pwm; // PWM̃nh

/*
 * [U[֐
 */

/*
 * CGI\[XR[hicontrol.cgij
 */
static int cgiControl(SOCKET s, int ContentLength, char *pArgs)
{
    char *buffer;
    char len;

    CONSOLE("CGIiRg[jJn\n");

    // obt@m
    buffer = (char*) mmBulkAlloc(ContentLength + 1);
    if (!buffer)
    {
        CONSOLE("obt@̊mۂɎs܂I\n");
        goto CGI_ERROR;
    }

    // POSTf[^擾
    len = recv(s, buffer, ContentLength, MSG_WAITALL);
    if (len < ContentLength)
    {
        CONSOLE("POSTf[^̎擾Ɏs܂I\n");
        goto CGI_ERROR;
    }
    buffer[ContentLength] = '\0';

    CONSOLE("POSTf[^:\n%s\n\n", buffer);

    // POSTf[^"toggle_LDM"̎ALDMON/OFF𔽓]
    if (strcmp(buffer, "toggle_LDM") == 0)
    {
        toggleLDM ^= 1;
    }

    // POSTf[^"changeChar_LDM"̎ALDM̕\ύX
    if (strcmp(buffer, "changeChar_LDM") == 0)
    {
        if (charLDM == 'A')
        {
            charLDM = 'B';
        }
        else
        {
            charLDM = 'A';
        }
    }

    // POSTf[^"servo_position=l"̎Aw肳ꂽl̊pxɐݒ
    if (strncmp(buffer, "servo_position=", 15) == 0)
    {
        // px̕𐮐ɕϊ
        int degree = atoi(buffer + 15);
        // px-90`90͈͓̔ɂ
        if (degree < -90)
            degree = -90;
        if (degree > 90)
            degree = 90;
        CONSOLE("degree: %d\n", degree);
        // pxɉf[eBi}CNbjݒ
        PWM_setDuty(pwm, (1.5 - 0.8 * degree / 90) * 1000);
    }

    httpSendStatusLine(s, HTTP_OK, CONTENT_TYPE_HTML);
    httpSendClientStr(s, CRLF);
    httpSendClientStr(s, "<!DOCTYPE html>\n"
                      "<html>\n"
                      "<head>\n"
                      "<meta charset='Shift_JIS'>\n"
                      "<title>WebT[õeXg</title>\n"
                      "</head>\n"
                      "<body>\n"
                      "CGĨeXg\n"
                      "</body>\n"
                      "</html>");

    CGI_ERROR:
    // obt@
    if (buffer)
        mmBulkFree(buffer);

    // 1Ԃ
    return 1;
}

/*
 * CGI\[XR[histatus.cgij
 */
static int cgiStatus(SOCKET s, int ContentLength, char *pArgs)
{
    char *buffer;
    char len;

    CONSOLE("CGIiԎ擾jJn\n");

    // obt@m
    buffer = (char*) mmBulkAlloc(ContentLength + 1);
    if (!buffer)
    {
        CONSOLE("obt@̊mۂɎs܂I\n");
        goto CGI_ERROR;
    }

    // POSTf[^擾
    len = recv(s, buffer, ContentLength, MSG_WAITALL);
    if (len < ContentLength)
    {
        CONSOLE("POSTf[^̎擾Ɏs܂I\n");
        goto CGI_ERROR;
    }
    buffer[ContentLength] = '\0';

    CONSOLE("POSTf[^:\n%s\n\n", buffer);

    // HTTPwb_̑M
    httpSendStatusLine(s, HTTP_OK, CONTENT_TYPE_HTML);
    httpSendClientStr(s, CRLF);

    // POSTf[^"get_sw"̎AĂXCb`ԐM
    if (strcmp(buffer, "get_sw") == 0)
    {
        // SW1Ă鎞̃eLXg
        if (GPIO_read(Board_BUTTON0) == 0)
        {
            httpSendClientStr(s, "SW1,");
        }
        // SW2Ă鎞̃eLXg𑗐M
        if(GPIO_read(Board_BUTTON1)==0) {
            httpSendClientStr(s, "SW2,");
        }
    }

    // POSTf[^"get_temp"̎Axf[^擾
    if (strcmp(buffer, "get_temp") == 0)
    {
        unsigned int data; // ADCf[^i[p
        char Temp[6]; // i[p
        int temp; // x̐Βl
        char minus; // x̕

        // xZTP1ADCl擾
        data = getADC(ADC_P1);
        CONSOLE("ADC: %d\n", data);

        // ADClxi0.1Pʁjɕϊ
        temp = (data * 3.3 * 1000 / 4096 - 600);

        // x̐Βlƕ擾
        minus = 0;
        if (temp < 0)
        {
            temp = -temp;
            minus = 1;
        }

        // Βl999ȏ̏ꍇ999i}99.9jƂ
        if (temp > 999)
            temp = 999;
        CONSOLE("Temp: %d\n", temp);

        // x̕쐬
        Temp[4] = '0' + temp % 10;          // _1
        Temp[3] = '.';                      // u.v
        Temp[2] = '0' + (temp / 10) % 10;   // 1̈
        if (temp / 100 > 1)                 // 10̈ʂѕ
        {
            Temp[1] = '0' + (temp / 100) % 10;
            Temp[0] = ' ';
            if (minus == 1)
                Temp[0] = '-';
        }
        else
        {
            Temp[1] = ' ';
            if (minus == 1)
                Temp[1] = '-';
            Temp[0] = ' ';
        }
        Temp[5] = '\0';

        // x̕𑗐M
        httpSendClientStr(s, Temp);
    }

    CGI_ERROR:
    // obt@
    if (buffer) mmBulkFree(buffer);

    // 1Ԃ
    return 1;
}

/*
 * [U[^XN
 */

/*
 * LED_Ń^XN
 */
Void task_LED(UArg arg0, UArg arg1)
{
    CONSOLE("LED_Ń^XN̊Jn\n");

    // LED1_
    GPIO_write(Board_LED0, Board_LED_ON);

    while (1)
    {
        // LED 0.5bɃgO
        SLEEP(500);
        GPIO_toggle(Board_LED0);
    }
}

/*
 * GLCD\^XN
 */
Void task_GLCD(UArg arg0, UArg arg1)
{
    char IP_Address[16]; // IPAhX\pz

    CONSOLE("GLCD\\^XN̊Jn\n");

    // eGLCDɕ\
    GLCD_str(0, 5, "HTTP SERVER");
    GLCD_str(1, 0, "IP Address:");

    // IPAhX擾܂ő҂
    while (IP == 0) SLEEP(100);

    // IPAhX𕶎ɕϊ
    NtIPN2Str(IP, IP_Address);

    // IPAhXGLCDɕ\
    GLCD_str(2, 6, IP_Address);

    while (1)
    {
        // ԑ҂
        SLEEP(1000);
    }
}

/*
 * FATFS^XN
 */
Void task_FATFS(UArg arg0, UArg arg1)
{
    SDSPI_Handle sdspiHandle; // SDJ[h}Eg̃nh
    SDSPI_Params sdspiParams; // SDJ[h}Eg̃p[^

    int i; // ėpJE^

    FILE *fd; // t@CfBXNv^
    FILE *fd_back; // obNAbvt@Cpt@CfBXNv^

    char buffer[128]; // t@Cǂݍ݁E݃obt@
    unsigned int bytes; // t@Cǂݍ݁E݃TCY

    char log[] = "fat:0:log.txt";           // Ot@C
    char log_back[] = "fat:0:log_b.txt";    // obNAbvt@C

    const int sampling = 60;   // xf[^̃TvO
    unsigned int data;         // ADCf[^i[pϐ
    int temp;                   // x̐Βl
    char minus;                 // x̕

    char Temp[6];   // x̕f[^i[pz
    char tstr[17];  // ̕f[^i[pz

    int restore ;   // obNAbvt@C̕tO

    CONSOLE("FATFS^XN̊Jn\n");

    // GLCDɕ\
    GLCD_str(6, 0, "Read text:");

    // ̃^XNSDJ[hgĂꍇ͎gI܂ő҂
    while (lockSD != 0) SLEEP(100);

    // SDJ[h̎gpJn
    lockSD = 1;

    // SDJ[h̐ݒp[^̏
    SDSPI_Params_init(&sdspiParams);

    // SDJ[h}Eg
    sdspiHandle = SDSPI_open(Board_SDSPI0, 0, &sdspiParams);

    // G[`FbN
    if (sdspiHandle == NULL)
    {
        CONSOLE("}EgɎs܂I\n");
        // GLCDɃG[bZ[W\
        GLCD_str(7, 0, "Mount error");
    }
    else
    {
        // t@CJ
        fd = fopen("fat:0:read.txt", "r");

        // G[`FbN
        if (fd == NULL)
        {
            CONSOLE("t@CJ܂łI\n");
            // GLCDɃG[bZ[W\
            GLCD_str(7, 0, "File open error");
        }
        else
        {
            // t@C當ǂݍ
            bytes = fread(buffer, 1, sizeof(buffer) -1, fd);
            buffer[bytes] = '\0';

            // GLCDɓǂݍ񂾕\
            GLCD_str(7, 0, buffer);

            // t@C
            fclose(fd);
        }

        // t@CJ
        fd = fopen("fat:0:write.txt", "w");

        // G[`FbN
        if (fd == NULL)
        {
            CONSOLE("t@CJ܂łI\n");

            // GLCDɃG[bZ[W\
            GLCD_str(7, 0, "File open error");
        }
        else
        {
            // ޕobt@ɃRs[
            strcpy(buffer, "Hello World!");

            // obt@̕t@Cɏ
            bytes = fwrite(buffer, 1, strlen(buffer), fd);

            // t@C
            fclose(fd);

            // ݃G[`FbN
            if (bytes == strlen(buffer))
            {
                GLCD_str(5, 0, "Write Successful");
            }
            else
            {
                GLCD_str(5, 0, "Write Error");
            }
        }

        // SDJ[h~
        SDSPI_close(sdspiHandle);
    }

    // SDJ[h̎gpI
    lockSD = 0;

    while (1)
    {
        /*
         * xf[^̍̕쐬
         */

        // xf[^̎擾i1b60񕪂̕ςj
        data = 0;
        for (i = 0; i < sampling; i++)
        {
            // ADCgp̏ꍇ͏I܂ő҂
            while (getADCStatus() == 1)
                SLEEP(10);
            // ADCl̎擾
            data += getADC(ADC_P1);
            CONSOLE("ADCϒl: %d (%d/%d)\n", data / (i + 1), i + 1, sampling);
            // 1b҂
            SLEEP(1000);
        }
        data /= sampling;

        // ADClxi0.1Pʁjɕϊ
        temp = (data * 3.3 * 1000 / 4096 - 600);

        // x̐Βlƕ擾
        if (temp < 0)
        {
            temp = -temp;
            minus = 1;
        }

        // Βl999ȏ̏ꍇ999i}99.9jƂ
        if (temp > 999)
            temp = 999;
        CONSOLE("Temp: %d\n", temp);

        // x̕쐬
        Temp[4] = '0' + temp % 10;          // _1
        Temp[3] = '.';                      // u.v
        Temp[2] = '0' + (temp / 10) % 10;   // 1̈
        if (temp / 100 > 1)                 // 10̈ʂѕ
        {
            Temp[1] = '0' + (temp / 100) % 10;
            Temp[0] = ' ';
            if (minus == 1)
                Temp[0] = '-';
        }
        else
        {
            Temp[1] = ' ';
            if (minus == 1)
                Temp[1] = '-';
            Temp[0] = ' ';
        }
        Temp[5] = '\0';

        /*
         * ݎ̍̕쐬
         */

        // [JԂ̎擾
        time_t timer = time(NULL);
        struct tm *t_st = localtime(&timer);

        // 𕶎ɕϊ
        tstr[0] = (t_st->tm_year + 1900) / 1000 + 0x30;
        tstr[1] = ((t_st->tm_year + 1900) / 100) % 10 + 0x30;
        tstr[2] = ((t_st->tm_year + 1900) / 10) % 10 + 0x30;
        tstr[3] = (t_st->tm_year + 1900) % 10 + 0x30;
        tstr[4] = '/';
        tstr[5] = (t_st->tm_mon + 1) / 10 + 0x30;
        tstr[6] = (t_st->tm_mon + 1) % 10 + 0x30;
        tstr[7] = '/';
        tstr[8] = (t_st->tm_mday) / 10 + 0x30;
        tstr[9] = (t_st->tm_mday) % 10 + 0x30;
        tstr[10] = ' ';
        tstr[11] = (t_st->tm_hour) / 10 + 0x30;
        tstr[12] = (t_st->tm_hour) % 10 + 0x30;
        tstr[13] = ':';
        tstr[14] = (t_st->tm_min) / 10 + 0x30;
        tstr[15] = (t_st->tm_min) % 10 + 0x30;
        tstr[16] = '\0';
        CONSOLE("log: %s %s\n", tstr, Temp);

        /*
         * Ot@C̃`FbNƃobNAbv
         */

        // tONA
        restore = 0;

        // ̃^XNSDJ[hgĂꍇ͎gI܂ő҂
        while (lockSD != 0) SLEEP(100);

        // SDJ[h̎gpJn
        lockSD = 1;

        // SDJ[h̐ݒp[^̏
        SDSPI_Params_init(&sdspiParams);

        // SDJ[h}Eg
        sdspiHandle = SDSPI_open(Board_SDSPI0, 0, &sdspiParams);

        // G[`FbN
        if (sdspiHandle == NULL)
        {
            // SDJ[h̎gpI
            lockSD = 0;
            CONSOLE("}EgɎs܂I\n");

            // GLCDɃG[bZ[W\
            GLCD_str(7, 0, "Mount error");
            continue;
        }

        // Ot@CǂݍݐpŊJ
        fd = fopen(log, "r");

        // obNAbvpt@CǂݍݐpŊJ
        fd_back = fopen(log_back, "r");

        // obNAbvt@C݂ꍇ
        if (fd_back)
        {
            if (!fd)
            {
                // Ot@C݂ȂAobNAbvt@C̕s
                restore = 1;
            }
            else
            {
                // et@C̖Ɉړ
                fseek(fd, 0L, SEEK_END);
                fseek(fd_back, 0L, SEEK_END);

                CONSOLE("O: %d, obNAbv: %d\n", ftell(fd), ftell(fd_back));

                // obNAbvt@Cقt@CTCY傫A
                // obNAbvt@C̕s
                if (ftell(fd_back) > ftell(fd))
                {
                    restore = 1;
                }
                // et@C̐擪Ɉړ
                fseek(fd, 0L, SEEK_SET);
                fseek(fd_back, 0L, SEEK_SET);
            }
        }

        // tOĂ鎞AobNAbvt@COt@CɃRs[A
        // łȂꍇAOt@CobNAbvt@CɃRs[
        if (restore)
        {
            CONSOLE("Ot@CjBJn\n");

            // Ot@CAݐpŊJ
            if (fd != NULL)
                fclose(fd);
            fd = fopen(log, "w");

            // obNAbvt@COt@CɃRs[
            if (fd)
            {
                while (1)
                {
                    if (feof(fd))
                        break;
                    fputc(fgetc(fd_back), fd);
                }
            }
            else
            {
                CONSOLE("Ot@C𕜋ł܂I\n");
            }
        }
        else
        {
            // Ot@C݂ꍇAOt@CobNAbv
            if (fd)
            {
                CONSOLE("Ot@CobNAbv\n");

                // obNAbvt@CAݐpŊJ
                if (fd_back != NULL) fclose(fd_back);
                fd_back = fopen(log_back, "w");

                // Ot@CobNAbvt@CɃRs[
                if (fd_back)
                {
                    i = 0;
                    while (1)
                    {
                        if (feof(fd))
                            break;
                        fputc(fgetc(fd), fd_back);
                    }
                }
                else
                {
                    CONSOLE("obNAbvł܂I\n");
                }
            }
        }

        // t@C
        if (fd != NULL)
            fclose(fd);
        if (fd_back != NULL)
            fclose(fd_back);

        /*
         * Ot@Cւ̉x̋L^
         */

        // Ot@CǋLpŊJ
        fd = fopen(log, "a");

        // G[`FbN
        if (!fd)
        {
            CONSOLE("Ot@CJ܂łI\n");

            // GLCDɃbZ[W\
            GLCD_str(5, 0, "File open error");
        }
        else
        {
            // ̕t@Cɏ
            bytes = fwrite(tstr, 1, strlen(tstr), fd);

            // 1󂯂
            if (fputc(' ', fd) != EOF)
                bytes++;

            // x̕t@Cɏ
            bytes += fwrite(Temp, 1, strlen(Temp), fd);

            // s
            bytes += fwrite("\r\n", 1, 2, fd);

            // t@C
            fclose(fd);

            // ݃G[`FbN
            if (bytes == strlen(tstr) + strlen(Temp) + 3)
            {
                GLCD_str(5, 0, "Write Successful");
            }
            else
            {
                GLCD_str(5, 0, "Write Error");
            }
        }

        /*
         * obNAbvt@C폜
         */
        remove(log_back);

        // SDJ[h~
        SDSPI_close(sdspiHandle);

        // SDJ[h̎gpI
        lockSD = 0;
    }
}

/*
 * LEDhbg}gNX^XN
 */
Void task_LDM(UArg arg0, UArg arg1)
{
    int i, j; // ėpJE^

    // LDMpsiAm[hj
    uint32_t LDM_A[] = { LDM_P1, LDM_P2, LDM_P3, LDM_P4, LDM_P5,LDM_P6, LDM_P7, LDM_P8 };

    // LDMpsiJ\[hj
    uint32_t LDM_C[] = { LDM_P9, LDM_P10, LDM_P11, LDM_P12, LDM_P13,LDM_P14, LDM_P15, LDM_P16 };

    // pJE^j0ɐݒ
    j = 0;
    while (1)
    {
        // S
        for (i = 0; i < 8; i++)
        {
            GPIO_write(LDM_A[i], 0);
            GPIO_write(LDM_C[i], 1);
        }

        // LEDhbg}gNX`ʂ̋x~
        if (toggleLDM != 1)
        {
            SLEEP(30);
            continue;
        }

        // ADCsLDM_P4̗΂
        if(getADCStatus()==1 && LDM_A[j]==LDM_P4) j++;

        if (charLDM == 'A')
        {
            // uAv̕`
            GPIO_write(LDM_A[j], 1);
            switch (j)
            {
            case 0:
                GPIO_write(LDM_C[6], 0);
                GPIO_write(LDM_C[7], 0);
                break;
            case 1:
                GPIO_write(LDM_C[4], 0);
                GPIO_write(LDM_C[5], 0);
                break;
            case 2:
                GPIO_write(LDM_C[2], 0);
                GPIO_write(LDM_C[3], 0);
                GPIO_write(LDM_C[5], 0);
                break;
            case 3:
                GPIO_write(LDM_C[0], 0);
                GPIO_write(LDM_C[1], 0);
                GPIO_write(LDM_C[5], 0);
                break;
            case 4:
                GPIO_write(LDM_C[2], 0);
                GPIO_write(LDM_C[3], 0);
                GPIO_write(LDM_C[5], 0);
                break;
            case 5:
                GPIO_write(LDM_C[4], 0);
                GPIO_write(LDM_C[5], 0);
                break;
            case 6:
                GPIO_write(LDM_C[6], 0);
                GPIO_write(LDM_C[7], 0);
                break;
            case 7:
                break;
            }
        }
        else if (charLDM == 'B')
        {
            // uBv̕`
            GPIO_write(LDM_A[j], 1);
            switch (j)
            {
            case 0:
                break;
            case 1:
                GPIO_write(LDM_C[0], 0);
                GPIO_write(LDM_C[1], 0);
                GPIO_write(LDM_C[2], 0);
                GPIO_write(LDM_C[3], 0);
                GPIO_write(LDM_C[4], 0);
                GPIO_write(LDM_C[5], 0);
                GPIO_write(LDM_C[6], 0);
                GPIO_write(LDM_C[7], 0);
                break;
            case 2:
                GPIO_write(LDM_C[0], 0);
                GPIO_write(LDM_C[3], 0);
                GPIO_write(LDM_C[7], 0);
                break;
            case 3:
                GPIO_write(LDM_C[0], 0);
                GPIO_write(LDM_C[3], 0);
                GPIO_write(LDM_C[7], 0);
                break;
            case 4:
                GPIO_write(LDM_C[0], 0);
                GPIO_write(LDM_C[3], 0);
                GPIO_write(LDM_C[7], 0);
                break;
            case 5:
                GPIO_write(LDM_C[1], 0);
                GPIO_write(LDM_C[2], 0);
                GPIO_write(LDM_C[4], 0);
                GPIO_write(LDM_C[5], 0);
                GPIO_write(LDM_C[6], 0);
                break;
            case 6:
                break;
            case 7:
                break;
            }
        }

        j++;
        if (j > 8) j = 0;

        SLEEP(1);
    }
}

/*
 * SNTP^XN
 */
Void task_SNTP(UArg arg0, UArg arg1);

/*
 * tbN֐
 */

Void netIPAddrHook(IPN IPAddr, uint IfIdx, uint fAdd)
{
    Task_Params taskParams;

    /* IPAhX̎擾 */
    IP = IPAddr;
    CONSOLE("IPAhX: %x\n", IP);

    CONSOLE("SNTP^XN̍쐬\n");
    Task_Params_init(&taskParams);
    taskParams.stackSize = TASK_SNTP_STACK;
    taskParams.stack = &taskSNTPStack;
    taskParams.priority = TASK_SNTP_PRIO;
    Task_construct(&taskSNTPStruct, (Task_FuncPtr)task_SNTP, &taskParams, NULL);
}

Void addWebFiles()
{
    SDSPI_Handle sdspiHandle; // SDJ[h}Eg̃nh
    SDSPI_Params sdspiParams; // SDJ[h}Eg̃p[^

    FILE *fd; // t@CfBXNv^

    static char page[4096]; // y[Wf[^̓ǂݍ݃obt@
    unsigned int bytesRead; // t@Cǂݍ݃TCY

    // ̃^XNSDJ[hgĂꍇ͎gI܂ő҂
    while (lockSD != 0) SLEEP(100);

    // SDJ[h̎gpJn
    lockSD = 1;

    // SDJ[h̐ݒp[^̏
    SDSPI_Params_init(&sdspiParams);

    // SDJ[h}Eg
    sdspiHandle = SDSPI_open(Board_SDSPI0, 0, &sdspiParams);

    // G[`FbN
    if (sdspiHandle == NULL)
    {
        CONSOLE("}EgɎs܂I\n");
        // GLCDɃG[bZ[W\
        GLCD_str(7, 0, "Mount error");
    }
    else
    {
        // t@CJ
        fd = fopen("fat:0:index.htm", "r");
        // G[`FbN
        if (fd == NULL)
        {
            CONSOLE("t@CJ܂łI\n");
            // GLCDɃG[bZ[W\
            GLCD_str(7, 0, "File open error");
        }
        else
        {
            // t@C當ǂݍ
            bytesRead = fread(page, 1, sizeof(page) - 1, fd);
            page[bytesRead] = '\0';
            CONSOLE("ǂݍ񂾃oCg: %d/%d\n", bytesRead, sizeof(page) - 1);

            // t@C
            fclose(fd);
        }
        // SDJ[h~
        SDSPI_close(sdspiHandle);
    }
    // SDJ[h̎gpI
    lockSD = 0;

    /* Webt@C̒ǉ */
    efs_createfile("index.html", strlen(page), (UINT8 *) page);
    efs_createfile("control.cgi", 0, (UINT8 *) cgiControl);
    efs_createfile("status.cgi", 0, (UINT8 *) cgiStatus);
}

Void removeWebFiles()
{
    /* Webt@C̍폜 */
    efs_destroyfile("index.html");
    efs_destroyfile("control.cgi");
    efs_destroyfile("status.cgi");
}

/*
 * C
 */

int main(void)
{
    Task_Params taskParams;
    PWM_Params pwmParams;

    // ݒ̏
    Seconds_set(0);

    // {[h̏ݒ
    CONSOLE("{[h̏ݒ\n");
    Board_initGeneral();
    Board_initGPIO();
    Board_initPWM();
    Board_initEMAC();
    Board_initSDSPI();

    // ADC̏ݒ
    initADC();

    // GLCD̏
    GLCD_init();

    // [U[ϐ̏l̐ݒ
    IP = 0;
    lockSD = 0;
    toggleLDM = 1;
    charLDM = 'A';

    // PWM̏ݒ
    // : PWMp[^̏
    PWM_Params_init(&pwmParams);
    // : PWM̎wi}CNbj
    pwmParams.period = 20*1000;
    // : PWMJ
    pwm = PWM_open(Servo_PWM, &pwmParams);
    // : PWM̃pXݒi}CNbj
    PWM_setDuty(pwm, 1.5*1000);

    // [U[^XN̍쐬
    CONSOLE("[U[^XN̍쐬\n");
    Task_Params_init(&taskParams);
    taskParams.stackSize = TASK_LED_STACK;
    taskParams.stack = &taskLEDStack;
    taskParams.priority = TASK_LED_PRIO;
    Task_construct(&taskLEDStruct, (Task_FuncPtr) task_LED, &taskParams, NULL);

    Task_Params_init(&taskParams);
    taskParams.stackSize = TASK_GLCD_STACK;
    taskParams.stack = &taskGLCDStack;
    taskParams.priority = TASK_GLCD_PRIO;
    Task_construct(&taskGLCDStruct, (Task_FuncPtr) task_GLCD, &taskParams, NULL);

    Task_Params_init(&taskParams);
    taskParams.stackSize = TASK_FATFS_STACK;
    taskParams.stack = &taskFATFSStack;
    taskParams.priority = TASK_FATFS_PRIO;
    Task_construct(&taskFATFSStruct, (Task_FuncPtr) task_FATFS, &taskParams, NULL);

    Task_Params_init(&taskParams);
    taskParams.stackSize = TASK_LDM_STACK;
    taskParams.stack = &taskLDMStack;
    taskParams.priority = TASK_LDM_PRIO;
    Task_construct(&taskLDMStruct, (Task_FuncPtr) task_LDM, &taskParams, NULL);

    // OS̋N
    CONSOLE("OS̋N\n");
    BIOS_start();

    return (0);
}
