/* XDCtools wb_t@C */
#include <xdc/std.h>
#include <xdc/runtime/System.h>

/* BIOS wb_t@C */
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Clock.h>

/* C wb_t@C */
#include <stdio.h>

/* hCowb_t@C */
#include <ti/drivers/GPIO.h>
#include <ti/drivers/SDSPI.h>

/* NDK wb_t@C */
#include <ti/ndk/inc/netmain.h>

/* {[hwb_t@C */
#include "Board.h"

/* GLCD wb_t@C */
#include "GLCD.h"

/* [U[`}N */
#define CONSOLE(...) do { System_printf(__VA_ARGS__); System_flush(); } while(0)
#define SLEEP(X) Task_sleep((X)*1000/Clock_tickPeriod)

/*
 * [U[^XN̗Dx
 */
#define TASK_LED_PRIO 1
#define TASK_GLCD_PRIO 9
#define TASK_FATFS_PRIO 10

/*
 * [U[X^bÑTCY
 */
#define TASK_LED_STACK 512
#define TASK_GLCD_STACK 512
#define TASK_FATFS_STACK 1024

/*
 * [U[^XN̍\
 */
Task_Struct taskLEDStruct;
Task_Struct taskGLCDStruct;
Task_Struct taskFATFSStruct;

/*
 * [U[^XÑX^bN
 */
Char taskLEDStack[TASK_LED_STACK];
Char taskGLCDStack[TASK_GLCD_STACK];
Char taskFATFSStack[TASK_FATFS_STACK];

/*
 * [U[ϐ
 */
uint32_t IP; // IPAhXi[pϐ

/*
 * [U[^XN
 */

/*
 * LED_Ń^XN
 */
Void task_LED(UArg arg0, UArg arg1)
{
    CONSOLE("LED_Ń^XN̊Jn\n");

    // LED1_
    GPIO_write(Board_LED0, Board_LED_ON);

    while (1)
    {
        // LED 0.5bɃgO
        SLEEP(500);
        GPIO_toggle(Board_LED0);
    }
}

/*
 * GLCD\^XN
 */
Void task_GLCD(UArg arg0, UArg arg1)
{
    char IP_Address[16]; // IPAhX\pz

    CONSOLE("GLCD\\^XN̊Jn\n");

    // eGLCDɕ\
    GLCD_str(0, 5, "HTTP SERVER");
    GLCD_str(1, 0, "IP Address:");

    // IPAhX擾܂ő҂
    while (IP == 0) SLEEP(100);

    // IPAhX𕶎ɕϊ
    NtIPN2Str(IP, IP_Address);

    // IPAhXGLCDɕ\
    GLCD_str(2, 6, IP_Address);

    while (1)
    {
        // ԑ҂
        SLEEP(1000);
    }
}

/*
 * FATFS^XN
 */
Void task_FATFS(UArg arg0, UArg arg1)
{
    SDSPI_Handle sdspiHandle; // SDJ[h}Eg̃nh
    SDSPI_Params sdspiParams; // SDJ[h}Eg̃p[^

    FILE *fd; // t@CfBXNv^

    char readBuffer[128]; // t@Cǂݍ݃obt@
    unsigned int bytesRead; // t@Cǂݍ݃TCY

    CONSOLE("FATFS^XN̊Jn\n");

    // GLCDɕ\
    GLCD_str(6, 0, "Read text:");

    // SDJ[h̐ݒp[^̏
    SDSPI_Params_init(&sdspiParams);

    // SDJ[h}Eg
    sdspiHandle = SDSPI_open(Board_SDSPI0, 0, &sdspiParams);

    // G[`FbN
    if (sdspiHandle == NULL)
    {
        CONSOLE("}EgɎs܂I\n");
        // GLCDɃG[bZ[W\
        GLCD_str(7, 0, "Mount error");
    }
    else
    {
        // t@CJ
        fd = fopen("fat:0:read.txt", "r");

        // G[`FbN
        if (fd == NULL)
        {
            CONSOLE("t@CJ܂łI\n");
            // GLCDɃG[bZ[W\
            GLCD_str(7, 0, "File open error");
        }
        else
        {
            // t@C當ǂݍ
            bytesRead = fread(readBuffer, 1, sizeof(readBuffer) -1, fd);
            readBuffer[bytesRead] = '\0';

            // GLCDɓǂݍ񂾕\
            GLCD_str(7, 0, readBuffer);

            // t@C
            fclose(fd);
        }

        // SDJ[h~
        SDSPI_close(sdspiHandle);
    }

    while (1)
    {
        // ԑ҂
        SLEEP(1000);
    }
}

/*
 * tbN֐
 */

Void netIPAddrHook(IPN IPAddr, uint IfIdx, uint fAdd)
{
    /* IPAhX̎擾 */
    IP = IPAddr;

    CONSOLE("IPAhX: %x\n", IP);
}

Void addWebFiles()
{
    static char page[] = "<!DOCTYPE html>\n"
            "<html>\n"
            "<head>\n"
            "<head>\n"
            "<meta charset='Shift_JIS'>\n"
            "<title>WebT[õeXg</title>\n"
            "<body>\n"
            "Hello, world!\n"
            "</body>\n"
            "</html>\n";

    /* Webt@C̒ǉ */
    efs_createfile("index.html", strlen(page), (UINT8 *) page);
}

Void removeWebFiles()
{
    /* Webt@C̍폜 */
    efs_destroyfile("index.html");
}

/*
 * C
 */

int main(void)
{
    Task_Params taskParams;

    // {[h̏ݒ
    CONSOLE("{[h̏ݒ\n");
    Board_initGeneral();
    Board_initGPIO();
    Board_initEMAC();
    Board_initSDSPI();

    // GLCD̏
    GLCD_init();

    // [U[ϐ̏l̐ݒ
    IP = 0;

    // [U[^XN̍쐬
    CONSOLE("[U[^XN̍쐬\n");
    Task_Params_init(&taskParams);
    taskParams.stackSize = TASK_LED_STACK;
    taskParams.stack = &taskLEDStack;
    taskParams.priority = TASK_LED_PRIO;
    Task_construct(&taskLEDStruct, (Task_FuncPtr) task_LED, &taskParams, NULL);

    Task_Params_init(&taskParams);
    taskParams.stackSize = TASK_GLCD_STACK;
    taskParams.stack = &taskGLCDStack;
    taskParams.priority = TASK_GLCD_PRIO;
    Task_construct(&taskGLCDStruct, (Task_FuncPtr) task_GLCD, &taskParams, NULL);

    Task_Params_init(&taskParams);
    taskParams.stackSize = TASK_FATFS_STACK;
    taskParams.stack = &taskFATFSStack;
    taskParams.priority = TASK_FATFS_PRIO;
    Task_construct(&taskFATFSStruct, (Task_FuncPtr) task_FATFS, &taskParams, NULL);

    // OS̋N
    CONSOLE("OS̋N\n");
    BIOS_start();

    return (0);
}
